@@ -4,7 +4,9 @@ import string
 from contextlib import contextmanager
 
 import tg
+from tg.render import render as tg_render
 from tg import expose
+import pylons
 from pylons import c
 from formencode import validators as fev
 from formencode import schema as fes
@@ -42,6 +44,8 @@ class Widget(fev.Validator):
     params=[]
     perform_validation = False
     template=None
+    exclude_names=[]
+    content_type='text/html'
 
     class __metaclass__(fev.Validator.__metaclass__):
         '''Make sure that the params lists of base classes are additive'''
@@ -62,12 +66,12 @@ class Widget(fev.Validator):
         if self.template:
             expose(self.template)(self)
 
-    def __call__(self, **kw):
-        context = dict(
+    def __call__(self, **context):
+        response = dict(
             (k, getattr(self, k))
             for k in self.params)
-        context.update(kw)
-        return context
+        response.update(context)
+        return response
 
     def validate(self, value, state):
         return self.to_python(value, state)
@@ -87,6 +91,12 @@ class ControllerWidget(Widget):
         else:
             self.validator = None
         self.decoration = controller.decoration
+        for content_type, (engine_name, template_name, exclude_names) \
+                in self.decoration.engines.iteritems():
+            if 'html' in content_type: break
+        self.template = '%s:%s' % (engine_name, template_name)
+        self.exclude_names = exclude_names
+        self.content_type = content_type
 
     def __call__(self, **kw):
         response = self.controller(**kw)
@@ -124,41 +134,37 @@ class WidgetInstance(object):
         self.context = context
         self.response = None
         self.parent = parent
-        self.dc = tg.controllers.DecoratedController()
 
     def __repr__(self):
         return 'I-%r' % self.widget_type
 
+    def _override_context(self):
+        if self.parent: return
+        widget_name = self.context.get('name', getattr(self.widget_type, 'name', None))
+        if not hasattr(c, 'validation_exception'): return
+        errors = c.validation_exception.unpack_errors()
+        value =  c.validation_exception.value
+        if widget_name:
+            if widget_name not in errors: return
+            errors = errors[widget_name]
+            value = value.get(widget_name, None)
+        self.context.update(value=value, errors=errors)
+
     def display(self):
-        if self.parent is None:
-            # Initialize name, value, and errors
-            if getattr(c, 'validation_exception', ''):
-                self.context['errors'] = c.validation_exception.unpack_errors()
-                self.context['value'] = c.validation_exception.value
-            else:
-                self.context['value'] = self.widget_type.from_python(self.context.get('value'), None)
-            if getattr(self.widget_type, 'name', None):
-                try:
-                    # Special handling for named forms
-                    self.context['value'] = self.context.get('value', {}).get(
-                        self.widget_type.name)
-                except AttributeError:
-                    log.warning('Apparently someone sent a named widget that was'
-                                ' not enclosed in a form (the value param was not'
-                                ' a dict)')
-                self.context['name'] = self.widget_type.name
-                    
+        # Run the widget on the context
+        self._override_context()
         response = self.widget_type(**self.context)
+        # Update the results
         if self.parent:
             # Underlay parent response fields
             d = dict(self.parent.response)
             d.update(response)
             response = d
         self.response = response
-        
+        # Render the response
         with _push_context(c, widget=self):
             try:
-                response = self.dc._render_response(self.widget_type, response)
+                response = self._render_response(response)
             except Exception, ex:
                 log.exception('Error rendering %s', self.widget_type)
                 raise
@@ -191,8 +197,8 @@ class WidgetInstance(object):
 
     def _context_for_str(self, name):
         result = dict(self.context)
-        if self.context.get('name'):
-            result['name'] = self.context['name'] + '.' + name
+        if self.response.get('name', None):
+            result['name'] = self.response['name'] + '.' + name
         else:
             result['name'] = name
         result['value'] = _safe_getitem(result, 'value', name)
@@ -201,19 +207,61 @@ class WidgetInstance(object):
 
     def _context_for_int(self, index):
         result = dict(self.context)
-        if self.context['name']:
-            result['name'] = '%s-%d' % (self.context['name'], index)
+        if self.response.get('name', None):
+            result['name'] = '%s-%d' % (self.response['name'], index)
         result['value'] = _safe_getitem(result, 'value', index)
         result['errors'] = _safe_getitem(result, 'errors', index)
         return result
 
+    def _render_response(self, response):
+        if ':' in self.widget_type.template:
+            engine_name, template_name = self.widget_type.template.split(':')
+        else:
+            config = pylons.configuration.config
+            engine_name = pylons.configuration.config.get("default_renderer", 'genshi')
+            template_name = self.widget_type.template
+        exclude_names = self.widget_type.exclude_names
+        if self.widget_type.content_type is not None:
+            pylons.response.headers['Content-Type'] = self.widget_type.content_type
+
+        # Save these objeccts as locals from the SOP to avoid expensive lookups
+        req = pylons.request._current_obj()
+        tmpl_context = pylons.tmpl_context._current_obj()
+
+        #if there is an identity, push it to the pylons template context
+        tmpl_context.identity = req.environ.get('repoze.who.identity')
+
+        # Setup the template namespace, removing anything that the user
+        # has marked to be excluded.
+        namespace = dict(tmpl_context=tmpl_context)
+        namespace.update(response)
+
+        for name in exclude_names:
+            namespace.pop(name)
+
+        # If we are in a test request put the namespace where it can be
+        # accessed directly
+        if req.environ.get('paste.testing'):
+            testing_variables = req.environ['paste.testing_variables']
+            testing_variables['namespace'] = namespace
+            testing_variables['template_name'] = template_name
+            testing_variables['controller_output'] = response
+
+        # Render the result.
+        result = tg_render(template_vars=namespace,
+                           template_engine=engine_name,
+                           template_name=template_name)
+
+        return result
+
 class WidgetsList(list):
     '''Simple class to let you create a list of widgets declaratively'''
     class __metaclass__(type):
         def __new__(meta, name, bases, dct):
             if bases == (list,):
                 return type.__new__(meta, name, bases, dct)
-            result = []
+            result = NameList()
+            result._index = dct
             for k,v in dct.iteritems():
                 if isinstance(v, Widget):
                     if v.name is None:
@@ -225,6 +273,14 @@ class WidgetsList(list):
             result.sort(key=lambda w:w._id)
             return result
 
+class NameList(list):
+
+    def __getitem__(self, index):
+        if isinstance(index, basestring):
+            return self._index[index]
+        return super(WidgetsList, self).__getitem__(index)
+
+        
 def _safe_getitem(dct, key, item):
     '''Return either dct[key][item],  dct[key].item, or None, whichever
     is appropriate
